@echo off
title Xeoma customizer


set SCRIPT_DIR_PATH=%~dp0
set SCRIPT_DIR_PATH=%SCRIPT_DIR_PATH:~0,-1%

set DIR_NAME_CUSTOM=custom
set DISTRIB_DIR_PATH=%SCRIPT_DIR_PATH%\distrib
set CUSTOM_DIR_PATH=%SCRIPT_DIR_PATH%\%DIR_NAME_CUSTOM%
set TOOLS_DIR_PATH=%SCRIPT_DIR_PATH%\tools
set ICON_DIR_PATH=%SCRIPT_DIR_PATH%\icon
set LOG_FILE_PATH=%SCRIPT_DIR_PATH%\customize.log

rem Main function - finds and processes all the executables from distrib folder
echo Xeoma customizing utility

echo Log started > "%LOG_FILE_PATH%"

for %%a in ("%DISTRIB_DIR_PATH%\win\*.exe")     do call :proc_customize         "%%a" "ico"
for %%a in ("%DISTRIB_DIR_PATH%\linux\*.tgz")   do call :proc_customize         "%%a" "ico"
for %%a in ("%DISTRIB_DIR_PATH%\mac\*.tgz")     do call :proc_customize         "%%a" "icns"
for %%a in ("%DISTRIB_DIR_PATH%\mac\*.dmg")     do call :proc_customize         "%%a" "icns"
for %%a in ("%DISTRIB_DIR_PATH%\android\*.apk") do call :proc_customize_android "%%a"

echo.

for %%a in ("%DISTRIB_DIR_PATH%\android\*.tmp") do (
    echo Rename "%%a" to "%%~na.apk"
    ren "%%a" "%%~na.apk"
)

echo Customizing finished!
exit /b


:proc_customize
    set FILE_PATH=%~1
    set ICON_EXT=%~2
    
    echo Processing file: '%FILE_PATH%'

    set ICO=
    for %%i in ("%ICON_DIR_PATH%\*.%ICON_EXT%") do set ICO=%%i

    set ICONPARAM=
    if not "%ICO%"=="" set ICONPARAM=-icon "%ICO%"
    
    "%TOOLS_DIR_PATH%\xeoma_rebrand.exe" %ICONPARAM% -resource "%CUSTOM_DIR_PATH%" -target "%FILE_PATH%" >> "%LOG_FILE_PATH%"
    
    if not %ERRORLEVEL% == 0 (
        call :printError "Customization error"
        exit /b 1
    )
exit /b


:proc_customize_android
    set APK_FILE_PATH=%~1
    set APK_DIR_PATH=%~dp1
    set APK_NAME=%~n1
    set RESOURCES_DIR_PATH=%TEMP%\customizebat
    set CUSTOMIZED_APK_PATH=%RESOURCES_DIR_PATH%\customized.apk
    echo    Start processing: %APK_FILE_PATH%

    if exist %RESOURCES_DIR_PATH% rmdir /Q /S %RESOURCES_DIR_PATH%
    md "%RESOURCES_DIR_PATH%"
    
    set ICON_FILE_PATH=
    for %%i in ("%ICON_DIR_PATH%\*.png") do set ICON_FILE_PATH=%%i

    if "%ICON_FILE_PATH%" == "" (
        call :printError "Path to icon file not specified"
        exit /b 1
    )

    if not exist "%ICON_FILE_PATH%" (
        call :printError "Not exist icon file: '%ICON_FILE_PATH%'"
        exit /b 1
    )

    set RESOURCE_ICONS_DIR_PATH=%RESOURCES_DIR_PATH%\icons
    set RESOURCE_CUSTOM_DIR_PATH=%RESOURCES_DIR_PATH%\%DIR_NAME_CUSTOM%
    if exist %RESOURCE_ICONS_DIR_PATH% rmdir /Q /S %RESOURCE_ICONS_DIR_PATH%
    md "%RESOURCE_ICONS_DIR_PATH%"
    md "%RESOURCE_CUSTOM_DIR_PATH%"

    rem Prepare icons ...
    "%SCRIPT_DIR_PATH%\tools\tools\pngresize\php\php.exe" -f "%SCRIPT_DIR_PATH%\tools\tools\pngresize\php\icon_resize.php" "%ICON_FILE_PATH%" "%RESOURCE_ICONS_DIR_PATH%"
    if not %ERRORLEVEL% == 0 (
        call :printError "Prepare icons error"
        exit /b 1
    )

    rem Copy 'custom' folder ...
    call :copyFile "%CUSTOM_DIR_PATH%\*.*" "%RESOURCE_CUSTOM_DIR_PATH%\*.*"
    if not %ERRORLEVEL% == 0 (
        call :printError "Copy file error"
        exit /b 1
    )

    rem Read product name ...
    set OEM_INFO_XML_PATH=%CUSTOM_DIR_PATH%\oem_info.xml
    
    if not exist "%OEM_INFO_XML_PATH%" (
        call :printError "Can't read product name: file not exists '%OEM_INFO_XML_PATH%'"
        exit /b 1
    )

    set PRODUCT_NAME=
    @for /F "tokens=1,2,3,4 delims=<>" %%a IN (%OEM_INFO_XML_PATH%) do if "%%b" == "ProductName" set PRODUCT_NAME=%%c && goto :exitloop
    :exitloop

    if "%PRODUCT_NAME%" == "" (
        call :printError "ProductName tag is not found at '%OEM_INFO_XML_PATH%'"
        exit /b 1
    )

    set PRODUCT_NAME=%PRODUCT_NAME:&apos;=\&apos;%

    echo    Product name: '%PRODUCT_NAME%'
    cscript /nologo "%SCRIPT_DIR_PATH%\tools\AndroidCustomizing.js" /CustomName:"%PRODUCT_NAME%" /ApkFilePath:"%APK_FILE_PATH%" /ResDirPath:"%RESOURCES_DIR_PATH%" /CustomizedApkPath:"%CUSTOMIZED_APK_PATH%" /LogFilePath:"%LOG_FILE_PATH%"

    if not %ERRORLEVEL% == 0 (
        call :printError "Android customization error"
        exit /b 1
    )
    
    if not exist "%CUSTOMIZED_APK_PATH%" (
        call :printError "Customized file not exists '%CUSTOMIZED_APK_PATH%'"
        exit /b 1
    )

    rem Share APK
    call :copyFile "%CUSTOMIZED_APK_PATH%" "%APK_DIR_PATH%\%APK_NAME%.%PRODUCT_NAME%.apk"
    if not %ERRORLEVEL% == 0 (
        call :printError "Share APK error"
        exit /b 1
    )

    if exist "%APK_FILE_PATH%"  del /F /Q "%APK_FILE_PATH%"
exit /b


:copyFile
    echo Copy: '%~1'
    echo   to: '%~2'
    copy /y /b "%~1" "%~2"
    
    if not %ERRORLEVEL% == 0 (
        call :printError "Copy ERROR"
        exit /b 1
    )
exit /b


:printError
    echo.
    echo.
    echo    ######## ########  ########   #######  ########  
    echo    ##       ##     ## ##     ## ##     ## ##     ## 
    echo    ##       ##     ## ##     ## ##     ## ##     ## 
    echo    ######   ########  ########  ##     ## ########  
    echo    ##       ##   ##   ##   ##   ##     ## ##   ##   
    echo    ##       ##    ##  ##    ##  ##     ## ##    ##  
    echo    ######## ##     ## ##     ##  #######  ##     ## 
    echo.
    echo.
    if not "%~1" == "" echo %~1
    echo.
    echo.
exit /b