// *****************************************************
var kForReading   = 1;
var kForWriting   = 2;
var kForAppending = 8;

var kWshRunning  = 0;
var kWshFinished = 1;
var kWshFailed   = 2;

var argLogFilePath, argCustomName, argApkFilePath, argPermissionsToRemove, argResourcesDirPath, argCustomizedApkPath;

var objLogFile, objFSO, objShell, objGlobal;

var objFSO   = new ActiveXObject("Scripting.FileSystemObject");
var objShell = new ActiveXObject("WScript.Shell");

checkArguments();
initGlobalVariables();

writeLogLine(":");
writeLogLine(":   ********************************************");
writeLogLine(":   *      Customize Xeoma to " + argCustomName);
writeLogLine(":   *");

var objCustom = prepareCustomize();

processDisassemble(objCustom);
processChangeResources(objCustom);
processRemovePermissions(objCustom);
processChangeNames(objCustom);
processAssemble(objCustom);
processZipTool(objCustom);
processSignApplication(objCustom);

writeLogLine(":   *");
writeLogLine(":   *      C U S T O M I Z E D   S U C C E S S F U L L Y");
writeLogLine(":   ********************************************");
writeLogLine(":");

WScript.quit(0);


//******************************************************************
function checkArguments()
{
    var objArguments = WScript.Arguments.Named;
    
    argLogFilePath = objArguments.Item("LogFilePath");
    if (isNullOrEmpty(argLogFilePath))
    {
        exitWithERROR("Customize ERROR: Not set log file path.");
    }

    // ATTENTION! Init log file HERE!
    objLogFile = objFSO.OpenTextFile(argLogFilePath, kForAppending, true);
    
    argCustomName = objArguments.Item("CustomName");
    if (isNullOrEmpty(argCustomName))
    {
        exitWithERROR("Customize ERROR: Not set custom name.");
    }
    
    argApkFilePath = objArguments.Item("ApkFilePath");
    if (isNullOrEmpty(argApkFilePath))
    {
        exitWithERROR("Customize ERROR: Not set APK file path.");
    }

    argResourcesDirPath = objArguments.Item("ResDirPath");
    if (isNullOrEmpty(argResourcesDirPath))
    {
        exitWithERROR("Customize ERROR: Not set resources dir path.");
    }

    if (! objFSO.FolderExists(argResourcesDirPath))
    {
        exitWithERROR("Customize ERROR: Not exist resources dir: '" + argResourcesDirPath + "'");
    }

    argCustomizedApkPath = objArguments.Item("CustomizedApkPath");
    if (isNullOrEmpty(argCustomizedApkPath))
    {
        exitWithERROR("Customize ERROR: Not set path to customized APK (path to result).");
    }

    if (objFSO.FileExists(argCustomizedApkPath))
    {
        objFSO.DeleteFile(argCustomizedApkPath);
    }

    argPermissionsToRemove = objArguments.Item("PermissionsToRemove");
}


// **********************************************************************
function initGlobalVariables()
{
    // local variable
    var strScriptDirPath = WScript.ScriptFullName.substring(0, WScript.ScriptFullName.lastIndexOf(WScript.ScriptName) - 1);

    objGlobal = new Object();
    objGlobal.strJavaBinDirPath          = strScriptDirPath   + "\\jdk1.8.0-251-min\\bin";
    
    objGlobal.strApkToolJarPath = strScriptDirPath + "\\tools\\apktool\\apktool.jar";
    checkFileExists(objGlobal.strApkToolJarPath);
    
    objGlobal.strXeomaZipalignExePath = strScriptDirPath + "\\tools\\zipalign.exe";
    checkFileExists(objGlobal.strXeomaZipalignExePath);
    
    objGlobal.strXeomaRebrandExePath = strScriptDirPath + "\\xeoma_rebrand.exe";
    checkFileExists(objGlobal.strXeomaRebrandExePath);
    
    objGlobal.strKeystoreJksPath = strScriptDirPath + "\\keystore.jks";
    checkFileExists(objGlobal.strKeystoreJksPath);
    
    objGlobal.strJavaExePath = objGlobal.strJavaBinDirPath + "\\java.exe";
    checkFileExists(objGlobal.strJavaExePath);
    
    objGlobal.strJarsignerExePath = objGlobal.strJavaBinDirPath + "\\jarsigner.exe";
    checkFileExists(objGlobal.strJarsignerExePath);

    objGlobal.strApkSignerPath = strScriptDirPath + "\\tools\\apksigner.jar";
    checkFileExists(objGlobal.strApkSignerPath);
}



// *
// *****************************************************************************
// *           C  U  S  T  O  M  I  Z  A  T  I  O  N                           *
// *****************************************************************************
function prepareCustomize()
{
    strWorkDirPath = objShell.ExpandEnvironmentStrings("%TEMP%") + "\\androidcustomizingjs";
    var objCustomVars = new Object();

    objCustomVars.strApkToolFrameworkDirPath             = objShell.ExpandEnvironmentStrings("%PUBLIC%") + "\\Documents\\apktoolsframework";
    objCustomVars.strCustomAppName                       = argCustomName;
    objCustomVars.strCustomAppNameWithoutSpaces          = argCustomName.split(" ").join("");
    objCustomVars.strCustomAppNameWithoutSpacesLowerCase = objCustomVars.strCustomAppNameWithoutSpaces.toLowerCase();
    objCustomVars.strCustomDirPath                       = argResourcesDirPath  + "\\custom";
    objCustomVars.strIconsDirPath                        = argResourcesDirPath  + "\\icons";
    objCustomVars.strDisassembleDirPath                  = strWorkDirPath       + "\\dis";
    objCustomVars.strAssetsDirPath                       = strWorkDirPath       + "\\dis\\assets";
    objCustomVars.strAppEditedApkPath                    = strWorkDirPath       + "\\edited.apk";
    objCustomVars.strAlignedApkPath                      = strWorkDirPath       + "\\aligned.apk";
    objCustomVars.objPermissonsToRemove                  = [];
    
    if (! isNullOrEmpty(argPermissionsToRemove))
    {
        objCustomVars.objPermissonsToRemove = argPermissionsToRemove.split(",");

        for (var index in objCustomVars.objPermissonsToRemove)
        {
            var strPermission = objCustomVars.objPermissonsToRemove[index];

            if ("string" == typeof(strPermission) && strPermission.length > 0)
            {
                objCustomVars.objPermissonsToRemove[index] = trim(strPermission);
            }
        }
    }

    clearDirectory(strWorkDirPath);
    clearDirectory(objCustomVars.strApkToolFrameworkDirPath);

    return objCustomVars;
}


function processDisassemble(objCustom)
{
    writeLogLine(":   *- Disassemble");

    var strCommand = objGlobal.strJavaExePath +
                    " -jar         \"" + objGlobal.strApkToolJarPath          + "\"" +
                    " --frame-path \"" + objCustom.strApkToolFrameworkDirPath + "\"" + 
                    " --output     \"" + objCustom.strDisassembleDirPath      + "\"" +
                    " d            \"" + argApkFilePath                       + "\"";
    executeCommand(strCommand, false);
}


function processChangeResources(objCustom)
{
    writeLogLine(":   *- Change resources");

    var strCommand = "\"" + objGlobal.strXeomaRebrandExePath + "\"" + 
                    " -packresource \"" + objCustom.strCustomDirPath + "\"" +
                    " -outdir       \"" + objCustom.strAssetsDirPath + "\"";
    
    executeCommand(strCommand, false);

    var strResourceOemFilePath = objCustom.strAssetsDirPath + "\\resource.oem";

    if (! objFSO.FileExists(strResourceOemFilePath))
    {
        exitWithERROR("Not created resources file: " + strResourceOemFilePath);
    }

    copyFile(objCustom.strIconsDirPath + "\\48x48.png"  , objCustom.strDisassembleDirPath + "\\res\\drawable-mdpi\\ic_launcher.png"  );
    copyFile(objCustom.strIconsDirPath + "\\72x72.png"  , objCustom.strDisassembleDirPath + "\\res\\drawable-hdpi\\ic_launcher.png"  );
    copyFile(objCustom.strIconsDirPath + "\\96x96.png"  , objCustom.strDisassembleDirPath + "\\res\\drawable-xhdpi\\ic_launcher.png" );
    copyFile(objCustom.strIconsDirPath + "\\144x144.png", objCustom.strDisassembleDirPath + "\\res\\drawable-xxhdpi\\ic_launcher.png");
}


function processChangeNames(objCustom)
{
    writeLogLine( ":   *- Change names (" + objCustom.strCustomAppName +")" );

    folderPaths = getSuitableSubdirectoriesPaths(objCustom.strDisassembleDirPath + "\\res\\", "values");

    for(var index in folderPaths)
    {
        var stringPath = folderPaths[index];

        if("string" == typeof(stringPath) && stringPath.length > 0 && objFSO.FileExists(stringPath + "\\strings.xml"))
        {
            replaceStrings(stringPath + "\\strings.xml", "Xeoma", objCustom.strCustomAppName);
        }
    }

    replaceStrings(objCustom.strDisassembleDirPath + "\\apktool.yml"             , "renameManifestPackage: null"   , "renameManifestPackage: com.felenasoft." + objCustom.strCustomAppNameWithoutSpacesLowerCase);
    replaceStrings(objCustom.strDisassembleDirPath + "\\AndroidManifest.xml"     , "XeomaService"                  , objCustom.strCustomAppNameWithoutSpaces + "Service");
    replaceStrings(objCustom.strDisassembleDirPath + "\\AndroidManifest.xml"     , "xeoma.provider"                , objCustom.strCustomAppNameWithoutSpacesLowerCase + ".provider");
    replaceStrings(objCustom.strDisassembleDirPath + "\\AndroidManifest.xml"     , "xeoma.firebaseinitprovider"    , objCustom.strCustomAppNameWithoutSpacesLowerCase + ".firebaseinitprovider");
    replaceStrings(objCustom.strDisassembleDirPath + "\\AndroidManifest.xml"     , "xeoma.androidx-startup"        , objCustom.strCustomAppNameWithoutSpacesLowerCase + ".androidx-startup");
}


function processRemovePermissions(objCustom)
{
    if (undefined != objCustom.objPermissonsToRemove && objCustom.objPermissonsToRemove.length > 0)
    {
        writeLogLine(":   *- Remove permissions");
    
        var strXmlFilePath = objCustom.strDisassembleDirPath + "\\AndroidManifest.xml";
        objFile = objFSO.OpenTextFile(strXmlFilePath, kForReading);
        var strFileContent = objFile.ReadAll();
        objFile.Close();
    
        var textLines = strFileContent.split("\r\n");
    
        if (textLines.length < 2)
        {
            exitWithERROR("ERROR: Wrong XML file: " + strXmlFilePath);
        }
    
        objFile = objFSO.OpenTextFile(strXmlFilePath, kForWriting);
    
        for (index in textLines)
        {
            var strLine = textLines[index];
    
            if (!isNeedRemove(strLine, objCustom.objPermissonsToRemove))
            {
                objFile.WriteLine(strLine);
            }
        }
    
        objFile.Close();
    }
    else
    {
        writeLogLine(":   *~ No permissions to remove");
    }
}


function processAssemble(objCustom)
{
    writeLogLine(":   *- Assemble (create APK)");

    var strCommand = objGlobal.strJavaExePath +
                    " -jar         \"" + objGlobal.strApkToolJarPath          + "\"" +
                    " --frame-path \"" + objCustom.strApkToolFrameworkDirPath + "\"" + 
                    " --output     \"" + objCustom.strAppEditedApkPath        + "\"" +
                    " b            \"" + objCustom.strDisassembleDirPath      + "\"";
    executeCommand(strCommand, false);

    if (!objFSO.FileExists(objCustom.strAppEditedApkPath))
    {
        exitWithERROR("ERROR assamble to " + objCustom.strAppEditedApkPath);
    }
}


function processSignApplication(objCustom)
{
    writeLogLine(":   *- Sign application");
    
    var strCommand = objGlobal.strJavaExePath +" -Xmx1024M -Xss1m -jar \"" + objGlobal.strApkSignerPath + "\"" +
                " sign --ks \"" + objGlobal.strKeystoreJksPath  + "\"" +
                " --ks-pass pass:felena1917" +
                " --out \"" + argCustomizedApkPath + "\"" +
                " \"" + objCustom.strAlignedApkPath + "\"";

    executeCommand(strCommand, false);
    
    if (!objFSO.FileExists(argCustomizedApkPath))
    {
        exitWithERROR("ERROR sign " + argCustomizedApkPath);
    }
    
    deleteFile(objCustom.strAlignedApkPath);
}


function processZipTool(objCustom)
{
    writeLogLine(":   *- APK inner content 4 byte aligned");

    var strCommand = objGlobal.strXeomaZipalignExePath + 
                    " -v 4" +
                    " " + objCustom.strAppEditedApkPath +
                    " " + objCustom.strAlignedApkPath;
    executeCommand(strCommand, false);

    deleteFile(objCustom.strAppEditedApkPath);
}


// *
// *****************************************************************************
// *                       H  E  L  P  E  R  S                                 *
// *****************************************************************************

function moveFile(strSrcPath, strDstPath)
{
    copyFile(strSrcPath, strDstPath, true);
    deleteFile(strSrcPath, true);
}


function copyFile(strSrcPath, strDstPath)
{
    if (!objFSO.FileExists(strSrcPath))
    {
        exitWithERROR("Can't copy file! Source file not exists: '" + strSrcPath + "'");
    }

    var parentFolderPath = objFSO.GetParentFolderName(strDstPath);

    if (!objFSO.FolderExists(parentFolderPath))
    {
        exitWithERROR("Can't copy file! Destination folder not exists: '" + parentFolderPath + "' ('" + strDstPath +"')");
    }

    objFSO.CopyFile(strSrcPath, strDstPath, true);
}


function deleteFile(strSrcPath)
{
    if (objFSO.FileExists(strSrcPath))
    {
        objFSO.DeleteFile(strSrcPath);
    }
}


function isNeedRemove(strLine, objPermissonsToRemove)
{
    bNeedRemove = false;
    
    if ("string" == typeof(strLine) && strLine.length > 0)
    {
        for (var index in objPermissonsToRemove)
        {
            var strPermission = objPermissonsToRemove[index];

            if ("string" == typeof(strPermission) && strPermission.length > 0 && strLine.search(strPermission) >= 0)
            {
                bNeedRemove = true;
                break;
            }
        }
    }

    return bNeedRemove;
}


function replaceStrings(srcFilePath, strSearch, strReplacement)
{
    objFile = objFSO.OpenTextFile(srcFilePath, kForReading);
    var strFileContent = objFile.ReadAll();
    objFile.Close();

    var strTextToWrite = strFileContent.split(strSearch).join(strReplacement);

    objFile = objFSO.OpenTextFile(srcFilePath, kForWriting);
    objFile.WriteLine(strTextToWrite);
    objFile.Close();
}



function getSuitableSubdirectoriesPaths(srcDirPath, srcTemplateName)
{
    var objFolder = objFSO.GetFolder(srcDirPath);
    var objSubfolders = [];

    var subfolders = new Enumerator(objFolder.SubFolders);

    for(; !subfolders.atEnd(); subfolders.moveNext())
    {
        var folderName = (subfolders.item()).name;

        if(folderName.indexOf(srcTemplateName) != -1)
        {
            objSubfolders.push((subfolders.item()).path);
        }
    }

    return objSubfolders;
}


// *******************************************************************************
function executeCommand(strCommand, needShowStdOut)
{
    var objShellExec = objShell.Exec(strCommand);

    while (objShellExec.Status == kWshRunning)
    {
        var strOutLine = objShellExec.StdOut.ReadLine();
        
        if ("" != strOutLine)
        {
            if (needShowStdOut)
            {
                writeRawLineToLog(strOutLine);
            }
            else
            {
                writeToLogFile(strOutLine);
            }
        }
        else
        {
            var strErrLine = objShellExec.StdErr.ReadLine();
    
            if ("" != strErrLine)
            {
                writeRawLineToLog("E: " + strErrLine);
            }
        }

        WScript.Sleep(50);
    }

    if (objShellExec.Status == kWshFailed || objShellExec.ExitCode != 0)
    {
        exitWithERROR(trimRight(objShellExec.StdErr.ReadAll()));
    }
    else if (needShowStdOut)
    {
        writeRawLineToLog(trimRight(objShellExec.StdOut.ReadAll()));
    }

    objShellExec = null;
}


function trimRight(strToTrim)
{
    return strToTrim.replace(/[ \t\r\n]+$/g, "");
}


function trim(strToTrim)
{
    return strToTrim.replace(/^[ \t\r\n]+|[ \t\r\n]+$/g, "");
}


function isNullOrEmpty(string)
{
    return undefined == string || "" == string;
}


function clearDirectory(strDirPath)
{
    if (objFSO.FolderExists(strDirPath))
    {
        objFSO.DeleteFolder(strDirPath);
    }

    objFSO.CreateFolder(strDirPath);
}


function checkFileExists(strFilePath)
{
    if (isNullOrEmpty(strFilePath))
    {
        exitWithERROR("Can't check file existance: file path is empty!");
    }

    if (! objFSO.FileExists(strFilePath))
    {
        exitWithERROR("FileNOT exists: '" + strFilePath + "'");
    }
}


function writeToLogFile(text)
{
    if (undefined != objLogFile)
    {
        objLogFile.writeLine(text);
    }
}


function writeRawLineToLog(text)
{
    WScript.StdOut.WriteLine(text);
    writeToLogFile(text);
}


function writeLogLine(text)
{
    writeRawLineToLog(" !   " + text);
}


function exitWithERROR(text)
{
    var message = "\n\n    E R R O R\n\n" + text + "\n\n";
    
    WScript.StdErr.WriteLine(message);
    writeToLogFile(message);

    WScript.quit(1);
}
